@@ -8,23 +8,32 @@
 *
 ***********************************************************************
 */
+#define __NO_VERSION__
+#include <linux/module.h>
+
 #include "nvhw.h"
 #include "nvwavout.h"
 #include "nvspdif.h"
 
-extern int strict_clocking;
 extern unsigned int clocking;
 
 static short AnalogLeftovers[6];
 static u32   AnalogLeftoverCount = 0;
 
-/* set playback sample rate */
+/**********************************************************************
+* set playback sample rate
+**********************************************************************/
 unsigned int Nvaudio_set_dac_rate(struct Nvaudio_state *state, unsigned int rate)
 {
     u32 new_rate;
     struct ac97_codec *codec=state->card->ac97_codec[0];
+    struct Nvaudio_card *card = state->card;
 
-    if(!(state->card->ac97_features&0x0001)) {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,9)  
+    u32 dacp = 0;
+#endif
+
+/*    if(!(state->card->ac97_features&0x0001)) {
         state->rate = clocking;
 
 #ifdef NV_DEBUG_PLAY
@@ -32,36 +41,66 @@ unsigned int Nvaudio_set_dac_rate(struct Nvaudio_state *state, unsigned int rate
                rate,clocking);
 #endif
         return clocking;
-    }
+    } */
 
-    if (rate > 48000)
-        rate = 48000;
+    if (rate > SAMPLERATE_48K)
+        rate = SAMPLERATE_48K;
     if (rate < 8000)
         rate = 8000;
     state->rate = rate;
 
-    /*
-     *  Adjust for misclocked crap
-     */
-    rate = ( rate * clocking)/48000;
-    if(strict_clocking && rate < 8000) {
-        rate = 8000;
-        state->rate = (rate * 48000)/clocking;
-    }
+    rate = ( rate * clocking)/clocking;
+    if(state->mapped) {
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)  
+        new_rate = ac97_set_dac_rate(codec, rate);
+#else
+    /* Power down the DAC */
+    dacp=Nvaudio_ac97_get(codec, AC97_POWER_CONTROL);
+    Nvaudio_ac97_set(codec, AC97_POWER_CONTROL, dacp|0x0200);
+    /* Load the rate and read the effective rate */
+    Nvaudio_ac97_set(codec, AC97_PCM_FRONT_DAC_RATE, rate);
+    new_rate=Nvaudio_ac97_get(codec, AC97_PCM_FRONT_DAC_RATE);
+    /* Power it back up */
+    Nvaudio_ac97_set(codec, AC97_POWER_CONTROL, dacp);
+#endif
+        if(new_rate != rate) {
+            state->rate = (new_rate * clocking)/clocking;
+        }
+    }else {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)  
+        /* set fixed sample rate now */
+        ac97_set_dac_rate(codec, SAMPLERATE_48K);
+#else
+        /* Power down the DAC */
+        dacp=Nvaudio_ac97_get(codec, AC97_POWER_CONTROL);
+        Nvaudio_ac97_set(codec, AC97_POWER_CONTROL, dacp|0x0200);
+        /* Load the rate and read the effective rate */
+        Nvaudio_ac97_set(codec, AC97_PCM_FRONT_DAC_RATE, SAMPLERATE_48K);
+        new_rate=Nvaudio_ac97_get(codec, AC97_PCM_FRONT_DAC_RATE);
+        /* Power it back up */
+        Nvaudio_ac97_set(codec, AC97_POWER_CONTROL, dacp);
+#endif
 
-    new_rate=ac97_set_dac_rate(codec, rate);
-    if(new_rate != rate) {
-        state->rate = (new_rate * 48000)/clocking;
     }
 #ifdef NV_DEBUG_PLAY
     printk("Nvaudio_set_dac_rate : asked for %d, got %d\n", rate, state->rate);
 #endif
-    rate = new_rate;
+
+    /* set the steps size Here */
+    card->ncurstepsize  = 1.0f;
+    card->ncurstepsize  = (float)rate / 48000.0f;
+    card->nCurTimeFract = 0.0f;
+    card->nCurTimeInt   = 0.0f;
+    card->nCurbuffStep  = 48000.0f / (float)rate;
+
     return state->rate;
 }
 
-/* get current playbackdma buffer pointer (byte offset from LBA),
-   called with spinlock held! */
+/**********************************************************************
+* get current playbackdma buffer pointer (byte offset from LBA),
+* called with spinlock held!
+**********************************************************************/
 unsigned Nvaudio_get_dma_addr(struct Nvaudio_state *state)
 {
     unsigned int civ, offset, port, port_picb, bytes = 2;
@@ -91,14 +130,37 @@ unsigned Nvaudio_get_dma_addr(struct Nvaudio_state *state)
         % dmabuf->dmasize);
 }
 
-/* stop playback (lock held) */
+/**********************************************************************
+* stop playback (lock held)
+**********************************************************************/
 void __stop_dac(struct Nvaudio_state *state)
 {
+    nodePtr curnode = NULL;
+    int count = 0;
     struct Nvaudio_card *card = state->card;
     state->enable &= ~DAC_RUNNING;
     outb(0, card->iobase + PO_CR);
+
+    if((card->outList) && (!state->mapped) && card->analogstate){
+        if(card->outList->wavoutcount) {
+            curnode = card->outList->wavoutnode;
+        }else{
+            if(card->outList->newnode) {
+                curnode = card->outList->newnode;
+            }
+        }
+        if(curnode) {
+            count = (curnode->size / card->numchannels) * card->numchannels;
+            count = curnode->size - count;
+            count = card->numchannels - (count >> 1);
+            count = (count << 1);
+            curnode->analogposn = count;
+            //printk("Setting analog position at stop %d \n",count);
+      }
+    }
     // wait for the card to acknowledge shutdown
     while( inb(card->iobase + PO_CR) != 0 ) ;
+
     // now clear any latent interrupt bits (like the halt bit)
 #ifdef NV_DEBUG_PLAY
     printk("Stopping DAC \n");
@@ -114,6 +176,9 @@ void stop_dac(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
+/**********************************************************************
+* start dac
+**********************************************************************/
 void __start_dac(struct Nvaudio_state *state)
 {
     int i =0;
@@ -121,7 +186,7 @@ void __start_dac(struct Nvaudio_state *state)
     struct Nvaudio_card *card = state->card;
     struct dmabuf *dmabuf = &state->dmabuffer;
 
-    if (dmabuf->count > 0 && state->ready && !state->enable &&
+    if (card->analogstate && dmabuf->count > 0 && state->ready && !state->enable &&
         (state->trigger & PCM_ENABLE_OUTPUT)) {
 
         /* Reset the Global Cnt back to Stereo 16ch and set the format 
@@ -156,6 +221,9 @@ void start_dac(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
+/**********************************************************************
+* update LVI
+**********************************************************************/
 void __Nvaudio_update_lvi(struct Nvaudio_state *state)
 {
     int x, port = 0;
@@ -169,8 +237,10 @@ void __Nvaudio_update_lvi(struct Nvaudio_state *state)
            (state->trigger & PCM_ENABLE_OUTPUT)) {
 
             outb((inb(port+PO_CIV)+1)&31, port+PO_LVI);
-            __start_dac(state);
-            while( (!(inb(port + PO_CR) & (0x10|0x04)))) ;
+            if(state->card->analogstate) {
+                __start_dac(state);
+                while( (!(inb(port + PO_CR) & (0x10|0x04)))) ;
+            }
         }
     }
 
@@ -193,7 +263,10 @@ void Nvaudio_update_lvi(struct Nvaudio_state *state)
     spin_unlock_irqrestore(&state->lock, flags);
 }
 
-/* update buffer manangement pointers, especially, dmabuf->count and dmabuf->hwptr */
+/**********************************************************************
+* update buffer manangement pointers, 
+* especially, dmabuf->count and dmabuf->hwptr
+**********************************************************************/
 void Nvaudio_update_ptr(struct Nvaudio_state *state)
 {
     unsigned hwptr = 0;
@@ -240,7 +313,9 @@ void Nvaudio_update_ptr(struct Nvaudio_state *state)
 
 }
 
-/* get the count of free space in the prds */
+/**********************************************************************
+* get the count of free space in the prds
+**********************************************************************/
 int Nvaudio_get_free_write_space(struct Nvaudio_state *state)
 {
     int free;
@@ -268,8 +343,9 @@ int Nvaudio_get_free_write_space(struct Nvaudio_state *state)
     return(free);
 }
 
-/* function to drain th existing data from dac */
-
+/**********************************************************************
+* function to drain th existing data from dac
+**********************************************************************/
 int drain_dac(struct Nvaudio_state *state, int signals_allowed)
 {
     unsigned long flags;
@@ -353,6 +429,9 @@ int drain_dac(struct Nvaudio_state *state, int signals_allowed)
     return 0;
 }
 
+/**********************************************************************
+* Nvaudio_channel_pcminterrupt
+**********************************************************************/
 void Nvaudio_channel_pcminterrupt(struct Nvaudio_card *card)
 {
     unsigned long port = 0;
@@ -422,16 +501,17 @@ void Nvaudio_channel_pcminterrupt(struct Nvaudio_card *card)
 #endif
 }
 
-/*
-   Function to check the speaker config and channel config and
-   determine to apply the downmix properly
-*/
+/**********************************************************************
+* Function to check the speaker config and channel config and
+*   determine to apply the downmix properly
+**********************************************************************/
 u32 Nvaudio_wavoutwrite(struct Nvaudio_card *card,const char* buffer,void * pDest, int cnt)
 {
     // Check what is the channel format and 
     //   Speaker selection
     u32 SampleCnt = 0;
     int loopend   = cnt >> 1;
+    struct Nvaudio_state *state = card->wavout;
 
     switch(card->numchannels) {
         case CHANNEL_51:    // 6 channel data check Spkr selection
@@ -440,14 +520,14 @@ u32 Nvaudio_wavoutwrite(struct Nvaudio_card *card,const char* buffer,void * pDes
 
                     case NV_SPKR_STEREO:
                         // Mix everything to Stereo
-                        SampleCnt = Nvaudio_MixDown5_1ToStereo(buffer,pDest,loopend,CHANNEL_51,FALSE);
+                        SampleCnt = Nvaudio_MixDown5_1ToStereo(buffer,state,loopend,CHANNEL_51,FALSE);
                         break;
                     case NV_SPKR_QUAD:
-                        SampleCnt = Nvaudio_MixDown5_1ToQuad(buffer,pDest,loopend,CHANNEL_51,FALSE);
+                        SampleCnt = Nvaudio_MixDown5_1ToQuad(buffer,state,loopend,CHANNEL_51,FALSE);
                         break;
                     case NV_SPKR_51:
                         // Copy  over the data
-                        SampleCnt = Nvaudio_MixNormal(buffer,pDest,loopend,CHANNEL_51,FALSE);
+                        SampleCnt = Nvaudio_MixNormal(buffer,state,loopend,CHANNEL_51,FALSE);
                         break;
                 }
             break;
@@ -458,16 +538,16 @@ u32 Nvaudio_wavoutwrite(struct Nvaudio_card *card,const char* buffer,void * pDes
 
                     case NV_SPKR_STEREO:
                         // Mix everything to Stereo 
-                        SampleCnt = Nvaudio_MixDownQuadToStereo(buffer,pDest,loopend,CHANNEL_QUAD,FALSE);
+                        SampleCnt = Nvaudio_MixDownQuadToStereo(buffer,state,loopend,CHANNEL_QUAD,FALSE);
                         break;
                     case NV_SPKR_QUAD:
                         // Copy over the data
-                         SampleCnt = Nvaudio_MixNormal(buffer,pDest,loopend,CHANNEL_QUAD,FALSE);
+                         SampleCnt = Nvaudio_MixNormal(buffer,state,loopend,CHANNEL_QUAD,FALSE);
                         break;
                     case NV_SPKR_51:
                         // Setting temporarily as QUAD now - Later we create new Center/Lfe channel
                         // Copy over the data
-                        SampleCnt = Nvaudio_MixNormal(buffer,pDest,loopend,CHANNEL_QUAD,FALSE);
+                        SampleCnt = Nvaudio_MixNormal(buffer,state,loopend,CHANNEL_QUAD,FALSE);
                         break;
                 }
             break;
@@ -478,16 +558,16 @@ u32 Nvaudio_wavoutwrite(struct Nvaudio_card *card,const char* buffer,void * pDes
                 switch(card->spkrselect) {
                     case NV_SPKR_STEREO:
                         // Copy Over Data
-                        SampleCnt = Nvaudio_MixNormal(buffer,pDest,loopend,CHANNEL_STEREO,FALSE);
+                        SampleCnt = Nvaudio_MixNormal(buffer,state,loopend,CHANNEL_STEREO,FALSE);
                         break;
                     case NV_SPKR_QUAD:
                         // Replicate data on Surrounds
-                        SampleCnt = Nvaudio_ReplicateStereotoRear(buffer,pDest,loopend,CHANNEL_STEREO);
+                        SampleCnt = Nvaudio_ReplicateStereotoRear(buffer,state,loopend,CHANNEL_STEREO);
                         break;
                     case NV_SPKR_51:
                         // Setting temporarily as Quad mode  now - with Replicate surrounds
                         //Later we create new Center/Lfe channel
-                        SampleCnt = Nvaudio_ReplicateStereotoRear(buffer,pDest,loopend,CHANNEL_STEREO);
+                        SampleCnt = Nvaudio_ReplicateStereotoRear(buffer,state,loopend,CHANNEL_STEREO);
                         break;
                 }
 
